@@ -11,6 +11,8 @@ use crate::result::MapmResult::*;
 use crate::template::fetch_template_config;
 use crate::template::Template;
 
+use crate::utils::copy::copy_dir_ignore_dots;
+
 use std::env;
 use std::fs;
 use std::path::Path;
@@ -106,28 +108,24 @@ pub fn parse_contest_yaml(yaml: &str, problem_dir: &Path, template_dir: &Path) -
                         contest_opt = None;
                     }
 
-                    return ContestResult {
+                    ContestResult {
                         contest: contest_opt,
                         contest_err: None,
                         problem_errs: problem_errs_opt,
-                    };
-                }
-                Fail(err) => {
-                    return ContestResult {
-                        contest: None,
-                        contest_err: Some(err),
-                        problem_errs: None,
-                    };
+                    }
                 }
+                Fail(err) => ContestResult {
+                    contest: None,
+                    contest_err: Some(err),
+                    problem_errs: None,
+                },
             }
         }
-        Err(err) => {
-            return ContestResult {
-                contest: None,
-                contest_err: Some(ContestErr(err.to_string())),
-                problem_errs: None,
-            };
-        }
+        Err(err) => ContestResult {
+            contest: None,
+            contest_err: Some(ContestErr(err.to_string())),
+            problem_errs: None,
+        },
     }
 }
 
@@ -139,23 +137,19 @@ pub fn fetch_contest(
     template_dir: &Path,
 ) -> ContestResult {
     match fs::read_to_string(contest_path) {
-        Ok(contest_yaml) => {
-            return parse_contest_yaml(&contest_yaml, problem_dir, template_dir);
-        }
-        Err(_) => {
-            return ContestResult {
-                contest: None,
-                contest_err: Some(ContestErr(
-                    [
-                        "Could not read contest from `",
-                        contest_path.to_str().unwrap(),
-                        "`",
-                    ]
-                    .concat(),
-                )),
-                problem_errs: None,
-            };
-        }
+        Ok(contest_yaml) => parse_contest_yaml(&contest_yaml, problem_dir, template_dir),
+        Err(_) => ContestResult {
+            contest: None,
+            contest_err: Some(ContestErr(
+                [
+                    "Could not read contest from `",
+                    contest_path.to_str().unwrap(),
+                    "`",
+                ]
+                .concat(),
+            )),
+            problem_errs: None,
+        },
     }
 }
 
@@ -171,9 +165,9 @@ impl Contest {
             }
         }
         if mapm_errs.len() > 0 {
-            return Some(mapm_errs);
+            Some(mapm_errs)
         } else {
-            return None;
+            None
         }
     }
     /// Compiles the contest given a template directory to query
@@ -184,7 +178,21 @@ impl Contest {
 
         let template_path = template_dir.join(&self.template.0);
         let build_dir = Path::new("build");
-        fs::create_dir(&build_dir).ok();
+        if !build_dir.is_dir() {
+            // If `build` exists but is not a directory, user intervention is required - so we panic.
+            if build_dir.exists() {
+                panic!(
+                    "`{}` exists in `{}` but is not a directory; please move it to fix this issue.",
+                    build_dir.to_str().unwrap(),
+                    cwd.to_str().unwrap()
+                );
+            }
+            fs::create_dir(&build_dir).expect(&format!(
+                "Could not create directory `{}` in `{}`",
+                build_dir.to_str().unwrap(),
+                cwd.to_str().unwrap()
+            ));
+        }
 
         if !template_path.exists() {
             return (
@@ -199,17 +207,13 @@ impl Contest {
                 results,
             );
         }
-        let options = fs_extra::dir::CopyOptions {
-            overwrite: true,
-            skip_exist: false,
-            buffer_size: 64000,
-            copy_inside: false,
-            content_only: true,
-            depth: 0,
-        };
-        fs_extra::dir::copy(template_path, "build", &options).unwrap();
 
-        env::set_current_dir(&build_dir).ok();
+        copy_dir_ignore_dots(&template_path, Path::new("build")).unwrap();
+
+        env::set_current_dir(&build_dir).expect(&format!(
+            "Could not set current directory to `{}`",
+            &build_dir.to_str().unwrap()
+        ));
 
         let mut headers = String::new();
 
@@ -226,47 +230,13 @@ impl Contest {
             );
         }
 
-        let mut problem_number = 0;
-
-        for (_, problem) in &self.problems {
-            problem_number += 1;
-            headers.push_str(
-                &[
-                    "\\expandafter\\def\\csname mapm@solcount@",
-                    &problem_number.to_string(),
-                    "\\endcsname{",
-                    &problem.solutions.len().to_string(),
-                    "}",
-                ]
-                .concat(),
-            );
-            for (key, val) in &problem.vars {
-                let filename =
-                    &["mapm-prob-", &problem_number.to_string(), "-", key, ".tex"].concat();
-                fs::write(filename, val).expect(&["Could not write to `", &filename, "`"].concat());
-            }
-            for (pos, map) in problem.solutions.iter().enumerate() {
-                for (key, val) in map {
-                    let solution_number = &pos + 1;
-                    let filename = &[
-                        "mapm-sol-",
-                        &problem_number.to_string(),
-                        "-",
-                        &solution_number.to_string(),
-                        "-",
-                        key,
-                        ".tex",
-                    ]
-                    .concat();
-                    fs::write(filename, val)
-                        .expect(&["Could not write to `", &filename, "`"].concat());
-                }
-            }
-        }
+        headers.push_str(&write_as_tex(
+            self.problems.iter().map(|p| p.1.clone()).collect(),
+        ));
 
         fs::write("mapm-headers.tex", headers).expect(
             &[
-                "Could not write `mapm-solcounts.tex` to directory `",
+                "Could not write `mapm-headers.tex` to directory `",
                 template_dir.to_str().unwrap(),
                 "`",
             ]
@@ -383,19 +353,26 @@ impl Contest {
             } else {
                 parsed_pdf_path.push_str(&unparsed_pdf_path);
             }
-            let latexmk = Command::new("latexmk").args([&format!("-pdflatex=`{}`", &self.template.1.engine), "-pdf", "-f", tex_path]).output().expect("Failed to execute latexmk.\nMake sure you have the latexmk script installed\nand make sure it is in your PATH.");
+            let latexmk = Command::new("latexmk").args([&format!("-pdflatex={}", &self.template.1.engine), "-pdf", "-f", tex_path]).output().expect("Failed to execute latexmk.\nMake sure you have the latexmk script installed\nand make sure it is in your PATH.");
             if latexmk.status.success() {
                 results.push(Ok(["Finished compiling ", &parsed_pdf_path].concat()));
                 let original_pdf_path = Path::new(tex_path).with_extension("pdf");
 
-                fs::rename(&original_pdf_path, &cwd.join(&parsed_pdf_path)).ok();
+                fs::rename(&original_pdf_path, &cwd.join(&parsed_pdf_path)).expect(&format!(
+                    "Could not rename `{}` to `{}`",
+                    &original_pdf_path.to_str().unwrap(),
+                    &parsed_pdf_path
+                ));
             } else {
                 results.push(Err(["Failed to compile ", &parsed_pdf_path].concat()));
             }
         }
 
-        env::set_current_dir(&cwd).ok();
+        env::set_current_dir(&cwd).expect(&format!(
+            "Failed to set current working directory to `{}`",
+            cwd.to_str().unwrap(),
+        ));
 
-        return (None, results);
+        (None, results)
     }
 }
