@@ -24,26 +24,17 @@ pub struct Template {
 pub fn fetch_template_config(template_name: &str, templates_dir: &Path) -> MapmResult<Template> {
     let template_path = templates_dir.join(template_name).join("config.yml");
     match fs::read_to_string(&template_path) {
-        Ok(template_yaml) => match parse_template_yaml(&template_yaml) {
-            Success(template) => {
-                return Success(template);
-            }
-            err => {
-                return err;
-            }
-        },
-        Err(_) => {
-            return Fail(TemplateErr(
-                [
-                    "Could not read template ",
-                    &template_name,
-                    " from `",
-                    &template_path.to_str().unwrap(),
-                    "`",
-                ]
-                .concat(),
-            ));
-        }
+        Ok(template_yaml) => parse_template_yaml(&template_yaml),
+        Err(_) => Fail(TemplateErr(
+            [
+                "Could not read template ",
+                &template_name,
+                " from `",
+                &template_path.to_str().unwrap(),
+                "`",
+            ]
+            .concat(),
+        )),
     }
 }
 
@@ -86,11 +77,7 @@ pub fn fetch_template_config(template_name: &str, templates_dir: &Path) -> MapmR
 
 pub fn parse_template_yaml(yaml: &str) -> MapmResult<Template> {
     match serde_yaml::from_str::<Template>(yaml) {
-        Ok(template) => {
-            return Success(template);
-        }
-        Err(err) => {
-            return Fail(TemplateErr(err.to_string()));
-        }
+        Ok(template) => Success(template),
+        Err(err) => Fail(TemplateErr(err.to_string())),
     }
 }
